"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArray = (this && this.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeEncode = exports.decodeWithOptions = exports.decode = exports.readMsg = exports.decodeId = exports.makeRecipes = void 0;
var util_1 = require("./util");
var get_1 = __importDefault(require("./get"));
var set_1 = __importDefault(require("./set"));
var add_1 = __importDefault(require("./add"));
var del_1 = __importDefault(require("./del"));
var operate_1 = __importDefault(require("./operate"));
var supported_1 = __importDefault(require("./supported"));
var proto_1 = __importDefault(require("./proto"));
var instances_1 = __importDefault(require("./instances"));
var notify_1 = __importDefault(require("./notify"));
var resolve_1 = __importDefault(require("./recipes/resolve"));
var operate_2 = __importDefault(require("./recipes/operate"));
var subscribe_1 = __importDefault(require("./recipes/subscribe"));
var commands = {
    GET: get_1.default,
    ADD: add_1.default,
    DELETE: del_1.default,
    GET_INSTANCES: instances_1.default,
    GET_SUPPORTED_DM: supported_1.default,
    GET_SUPPORTED_PROTO: proto_1.default,
    NOTIFY: notify_1.default,
    OPERATE: operate_1.default,
    SET: set_1.default,
};
var recipes = [resolve_1.default, operate_2.default, subscribe_1.default];
var makeRecipes = function (call, on) {
    return recipes.reduce(function (acc, _a) {
        var _b;
        var make = _a.make, name = _a.name;
        return (__assign(__assign({}, acc), (_b = {}, _b[name] = make(call, on), _b)));
    }, {});
};
exports.makeRecipes = makeRecipes;
var decodeId = function (data) { return String(data); };
exports.decodeId = decodeId;
var unkownErr = function (msg) { return ["error", "", msg]; };
var readMsg = function (proto, data) {
    var record = proto.rootRecord.lookupType("usp_record.Record");
    var decodedRecord = record.decode(data);
    var msg = proto.rootMsg.lookupType("usp.Msg");
    var decodedMsg = msg.decode(decodedRecord.noSessionContext.payload);
    return JSON.parse(JSON.stringify(decodedMsg)); // forces conversions
};
exports.readMsg = readMsg;
var decode = function (parsedMsg) {
    var err = util_1.searchParent(parsedMsg, "errMsg") || null;
    var command = util_1.extractCommand(parsedMsg);
    var foundId = util_1.search(parsedMsg, "msgId");
    // if id is formatted by me (command@) then use, otherwise check for sub id
    var id = foundId.includes("@")
        ? foundId
        : util_1.search(parsedMsg, "subscriptionId") || null;
    // if command is unkown
    if (!command)
        return unkownErr(parsedMsg);
    if (err)
        return [id, null, err, command];
    var cmd = commands[command] || null;
    if (!cmd)
        return unkownErr(parsedMsg);
    var _a = cmd.decode(parsedMsg), decodedData = _a[0], decodedId = _a[1], decodedErr = _a[2];
    return [decodedId || id, decodedData, decodedErr || err, command];
};
exports.decode = decode;
var decodeWithOptions = function (parsedMsg, cmdType, options) {
    var cmd = commands[cmdType] || null;
    if (!cmd)
        return unkownErr(parsedMsg);
    var decodedData = cmd.decode(parsedMsg, options)[0];
    return decodedData;
};
exports.decodeWithOptions = decodeWithOptions;
var makeEncode = function (proto, options) {
    return function (command, args) {
        var cmd = commands[command] || null;
        if (!cmd)
            return ["error", null, "Uknown command: " + command];
        return __spreadArray([], convert(proto, cmd.encode(args), options));
    };
};
exports.makeEncode = makeEncode;
var convert = function (proto, msg, bufferOptions) {
    var id = msg.header.msgId;
    msg.header.msgType = proto.header.MsgType[msg.header.msgType];
    var converted = _convert(proto, msg);
    if (isError(converted))
        return [id, null, converted];
    var encoded = proto.rootMsg
        .lookupType("usp.Msg")
        .encode(converted)
        .finish();
    var buffer = util_1.makeBuffer(proto.rootRecord, encoded, bufferOptions || {});
    return [id, buffer, null];
};
var isError = function (o) { return typeof o == "string"; };
var internalKeys = ["lookup"];
var isInternal = function (key) { return internalKeys.includes(key); };
var makePayload = function (items, isArr) {
    return items
        .filter(function (_a) {
        var k = _a[0];
        return !isInternal(k);
    })
        .reduce(function (acc, _a) {
        var _b;
        var k = _a[0], v = _a[1];
        return isArr
            ? __spreadArray(__spreadArray([], acc), [v]) : __assign(__assign({}, acc), (_b = {}, _b[k] = v, _b));
    }, isArr ? [] : {});
};
var isStringArray = function (obj) {
    return Array.isArray(obj) && obj.every(function (v) { return typeof v === "string"; });
};
var needsConversion = function (v) { return typeof v === "object" && !isStringArray(v); };
var _convert = function (proto, value) {
    var skip = value.lookup === undefined;
    var lookup = "usp." + value.lookup;
    var item = skip ? null : proto.rootMsg.lookupType(lookup);
    var simpleValues = Object.entries(value).filter(function (_a) {
        var v = _a[1];
        return !needsConversion(v);
    });
    var toConvert = Object.entries(value).filter(function (_a) {
        var v = _a[1];
        return needsConversion(v);
    });
    var converted = toConvert.map(function (_a) {
        var k = _a[0], v = _a[1];
        return [
            k,
            _convert(proto, v),
        ];
    });
    var err = converted.find(function (_a) {
        var v = _a[1];
        return isError(v);
    });
    if (err)
        return err[1];
    var total = converted.concat(simpleValues);
    var payload = makePayload(total, Array.isArray(value));
    var payloadErr = item === null || item === void 0 ? void 0 : item.verify(payload);
    if (payloadErr)
        return payloadErr;
    return item ? item.create(payload) : payload;
};
