import { extractCommand, makeBuffer, search, searchParent } from "./util.js";
import get from "./get.js";
import set from "./set.js";
import add from "./add.js";
import del from "./del.js";
import operate from "./operate.js";
import supported from "./supported.js";
import proto from "./proto.js";
import instances from "./instances.js";
import notify from "./notify.js";
import resolve from "./recipes/resolve.js";
import operateRecipe from "./recipes/operate.js";
import subscribe from "./recipes/subscribe.js";
const commands = {
GET: get,
ADD: add,
DELETE: del,
GET_INSTANCES: instances,
GET_SUPPORTED_DM: supported,
GET_SUPPORTED_PROTO: proto,
NOTIFY: notify,
OPERATE: operate,
SET: set,
};
const recipes = [resolve, operateRecipe, subscribe];
export const makeRecipes = (call, on) => recipes.reduce((acc, { make, name }) => ({ ...acc, [name]: make(call, on) }), {});
export const decodeId = (data) => String(data);
const unkownErr = (msg) => ["error", "", msg];
export const readMsg = (proto, data) => {
const record = proto.rootRecord.lookupType("usp_record.Record");
const decodedRecord = record.decode(data);
const msg = proto.rootMsg.lookupType("usp.Msg");
const decodedMsg = msg.decode(decodedRecord.noSessionContext.payload);
return JSON.parse(JSON.stringify(decodedMsg)); // forces conversions
};
export const decode = (parsedMsg) => {
const err = searchParent(parsedMsg, "errMsg") || null;
const command = extractCommand(parsedMsg);
const foundId = search(parsedMsg, "msgId");
// if id is formatted by me (command@) then use, otherwise check for sub id
const id = foundId.startsWith(command + "@")
? foundId
: search(parsedMsg, "subscriptionId") || null;
// if command is unkown
if (!command)
return unkownErr(parsedMsg);
if (err)
return [id, null, err, command];
const cmd = commands[command] || null;
if (!cmd)
return unkownErr(parsedMsg);
const [ddata, did, derr] = cmd.decode(parsedMsg);
return [did || id, ddata, derr || err, command];
};
export const makeEncode = (proto, options) => (command, args) => {
const cmd = commands[command] || null;
if (!cmd)
return ["error", null, `Uknown command: ${command}`];
return [...convert(proto, cmd.encode(args), options)];
};
const convert = (proto, msg, bufferOptions) => {
const id = msg.header.msgId;
msg.header.msgType = proto.header.MsgType[msg.header.msgType];
const converted = _convert(proto, msg);
if (isError(converted))
return [id, null, converted];
const encoded = proto.rootMsg
.lookupType("usp.Msg")
.encode(converted)
.finish();
const buffer = makeBuffer(proto.rootRecord, encoded, bufferOptions || {});
return [id, buffer, null];
};
const isError = (o) => typeof o == "string";
const internalKeys = ["lookup"];
const isInternal = (key) => internalKeys.includes(key);
const makePayload = (items, isArr) => items
.filter(([k]) => !isInternal(k))
.reduce((acc, [k, v]) => isArr
? [...acc, v]
: {
...acc,
[k]: v,
}, isArr ? [] : {});
const isStringArray = (obj) => Array.isArray(obj) && obj.every((v) => typeof v === "string");
const needsConversion = (v) => typeof v === "object" && !isStringArray(v);
const _convert = (proto, value) => {
const skip = value.lookup === undefined;
const lookup = "usp." + value.lookup;
const item = skip ? null : proto.rootMsg.lookupType(lookup);
const simpleValues = Object.entries(value).filter(([, v]) => !needsConversion(v));
const toConvert = Object.entries(value).filter(([, v]) => needsConversion(v));
const converted = toConvert.map(([k, v]) => [
k,
_convert(proto, v),
]);
const err = converted.find(([, v]) => isError(v));
if (err)
return err[1];
const total = converted.concat(simpleValues);
const payload = makePayload(total, Array.isArray(value));
const payloadErr = item?.verify(payload);
if (payloadErr)
return payloadErr;
return item ? item.create(payload) : payload;
};
